@@ -20,6 +20,9 @@ from . import __version__
 # Image to use
 IMAGE = "cs50/cli"
 
+# Label to use
+LABEL = "cli50"
+
 # Internationalization
 t = gettext.translation("cli50", pkg_resources.resource_filename("cli50", "locale"), fallback=True)
 t.install()
@@ -35,9 +38,11 @@ def main():
     parser.add_argument("-d", "--dotfile", action="append", default=[],
                         help=_("dotfile in your $HOME to mount read-only in container's $HOME"), metavar="DOTFILE")
     parser.add_argument("-f", "--fast", action="store_true", help=_("skip autoupdate"))
+    parser.add_argument("-i", "--image", default=IMAGE, help=_("start IMAGE, else {}").format(IMAGE), metavar="IMAGE")
+    parser.add_argument("-j", "--jekyll", action="store_true", help=_("serve Jekyll site"))
     parser.add_argument("-l", "--login", const=True, default=False, help=_("log into CONTAINER"), metavar="CONTAINER", nargs="?")
     parser.add_argument("-S", "--stop", action="store_true", help=_("stop any containers"))
-    parser.add_argument("-t", "--tag", default="latest", help=_("start {}:TAG, else {}:latest").format(IMAGE, IMAGE), metavar="TAG")
+    parser.add_argument("-u", "--update", action="store_true", help=_("update only"))
     parser.add_argument("-V", "--version", action="version", version="%(prog)s {}".format(__version__) if __version__ else "Locally installed.")
     parser.add_argument("directory", default=os.getcwd(), metavar="DIRECTORY", nargs="?", help=_("directory to mount, else $PWD"))
     args = vars(parser.parse_args())
@@ -52,6 +57,10 @@ def main():
         except AssertionError:
             print(_("A newer version is available. Run `pip3 install --upgrade cli50` to upgrade."))
 
+    # Mutually exclusive arguments
+    if args["fast"] and args["update"]:
+        sys.exit("Cannot use -f/--fast and -u/--update together.")
+
     # Check if Docker installed
     if not shutil.which("docker"):
         parser.error(_("Docker not installed."))
@@ -64,30 +73,6 @@ def main():
     except subprocess.TimeoutExpired:
         sys.exit("Docker not responding.")
 
-    # Reference to use
-    reference = f"{IMAGE}:{args['tag']}"
-
-    # Stop containers
-    if args["stop"]:
-        try:
-            stdout = subprocess.check_output([
-                "docker", "ps",
-                "--all",
-                "--format", "{{.ID}}\t{{.Image}}"
-            ]).decode("utf-8")
-            for line in stdout.rstrip().splitlines():
-                ID, Image = line.split("\t")
-                if Image == image:
-                    subprocess.check_call(["docker", "stop", "--time", "0", ID])
-            sys.exit(0)
-        except subprocess.CalledProcessError:
-            sys.exit(1)
-
-    # Ensure directory exists
-    directory = os.path.realpath(args["directory"])
-    if not os.path.isdir(directory):
-        parser.error(_("{}: no such directory").format(args['directory']))
-
     # Log into container
     if args["login"]:
 
@@ -107,7 +92,7 @@ def main():
                 "docker", "ps",
                 "--all",
                 "--filter", "status=running",
-                "--format", "{{.ID}}\t{{.Image}}\t{{.RunningFor}}\t{{.Mounts}}",
+                "--format", "{{.ID}}\t{{.Image}}\t{{.RunningFor}}\t{{.Status}}\t{{.Mounts}}",
                 "--no-trunc"
             ]).decode("utf-8")
         except subprocess.CalledProcessError:
@@ -115,21 +100,24 @@ def main():
         else:
             containers = []
             for line in stdout.rstrip().splitlines():
-                ID, Image, RunningFor, *Mounts = line.split("\t")
+                ID, Image, RunningFor, Status, *Mounts = line.split("\t")
                 Mounts = Mounts[0].split(",") if Mounts else []
                 Mounts = [Mount for Mount in Mounts if not re.match(r"^[0-9a-fA-F]{64}$", Mount)]  # Ignore hashes
-                containers.append((ID, Image, RunningFor.lower(), Mounts))
+                containers.append((ID, Image, RunningFor.lower(), Status.lower(), Mounts))
         if not containers:
             sys.exit("No containers are running.")
 
         # Ask whether to use a running container
-        for ID, Image, RunningFor, Mounts in containers:
+        for ID, Image, RunningFor, Status, Mounts in containers:
             while True:
-                prompt = _("Log into {}, started {}").format(Image, RunningFor)
+                prompt = _("Log into {}, created {}, {},").format(Image, RunningFor, Status)
                 if Mounts:
                     prompt += _(" with {} mounted").format(inflect.engine().join(Mounts))
                 prompt += "? [Y] "
-                stdin = input(prompt)
+                try:
+                    stdin = input(prompt)
+                except EOFError:
+                    break
                 if re.match("^\s*(?:y|yes)?\s*$", stdin, re.I):
                     try:
                         print(ports(ID))
@@ -143,42 +131,43 @@ def main():
         else:
             sys.exit(0)
 
+    # Stop containers
+    if args["stop"]:
+        try:
+            stdout = subprocess.check_output([
+                "docker", "ps",
+                "--all",
+                "--filter", f"label={LABEL}",
+                "--format", "{{.ID}}"
+            ]).decode("utf-8")
+
+            for ID in stdout.rstrip().splitlines():
+                subprocess.check_call(["docker", "stop", "--time", "0", ID])
+
+            sys.exit(0)
+        except subprocess.CalledProcessError:
+            sys.exit(1)
+
+    # Update only
+    if args["update"]:
+        pull(args["image"])
+        sys.exit(0)
+
+    # Ensure directory exists
+    directory = os.path.realpath(args["directory"])
+    if not os.path.isdir(directory):
+        parser.error(_("{}: no such directory").format(args['directory']))
+
     # Check for newer image
     if not args["fast"]:
-        try:
-
-            # Get digest of local image, if any
-            digest = subprocess.check_output(["docker", "inspect", "--format", "{{index .RepoDigests 0}}", f"{reference}"],
-                                             stderr=subprocess.DEVNULL).decode("utf-8").rstrip()
-
-            # Get digest of latest image
-            # https://hackernoon.com/inspecting-docker-images-without-pulling-them-4de53d34a604
-            # https://stackoverflow.com/a/35420411/5156190
-            response = requests.get(f"https://auth.docker.io/token?scope=repository:{IMAGE}:pull&service=registry.docker.io")
-            token = response.json()["token"]
-            response = requests.get(f"https://registry-1.docker.io/v2/{IMAGE}/manifests/{args['tag']}", headers={
-                "Accept": "application/vnd.docker.distribution.manifest.v2+json",
-                "Authorization": f"Bearer {token}"})
-
-            # Pull latest if digests don't match
-            assert digest == f"{IMAGE}@{response.headers['Docker-Content-Digest']}"
-
-        except (AssertionError, requests.exceptions.ConnectionError, subprocess.CalledProcessError):
-
-            # Pull image
-            try:
-                subprocess.check_call(["docker", "pull", reference], stderr=subprocess.DEVNULL)
-
-            # But don't prevent usage if pull fails (e.g., because no internet)
-            except subprocess.CalledProcessError:
-                pass
+        pull(args["image"])
 
     # Options
     options = ["--detach",
                "--interactive",
-               "--publish-all",
+               "--label", LABEL,
                "--rm",
-               "--security-opt", "seccomp=unconfined",  # https://stackoverflow.com/q/35860527#comment62818827_35860527
+               "--security-opt", "seccomp=unconfined",  # https://stackoverflow.com/q/35860527#comment62818827_35860527, https://github.com/apple/swift-docker/issues/9#issuecomment-328218803
                "--tty",
                "--volume", directory + ":/mnt",
                "--workdir", "/mnt"]
@@ -198,11 +187,31 @@ def main():
             sys.exit(_("{}: Not a dotfile").format(dotfile))
         options += ["--volume", "{}:/home/ubuntu/{}:ro".format(dotfile, dotfile[len(home):])]
 
+    # Default CMD
+    cmd = ["bash", "--login"]
+
+    # Serve Jekyll site
+    if args["jekyll"]:
+        cmd += ["-c", "bundle install && bundle exec jekyll serve --host 0.0.0.0 --port 8080"]
+
     # Mount directory in new container
     try:
 
         # Spawn container
-        container = subprocess.check_output(["docker", "run"] + options + [reference, "bash", "--login"]).decode("utf-8").rstrip()
+        try:
+
+            # Publish container's ports to the host
+            # https://stackoverflow.com/a/952952/5156190
+            container = subprocess.check_output(["docker", "run"] + options +
+                                                [item for sublist in [['--publish', f'{port}:{port}'] for port in (8080, 8081, 8082)] for item in sublist] +
+                                                [f"{args['image']}"] + cmd, stderr=subprocess.STDOUT).decode("utf-8").rstrip()
+
+        except subprocess.CalledProcessError:
+
+            # Publish all exposed ports to random ports
+            container = subprocess.check_output(["docker", "run"] + options +
+                                                ["--publish-all"] +
+                                                [f"{args['image']}"] + cmd).decode("utf-8").rstrip()
 
         # List port mappings
         print(ports(container))
@@ -251,5 +260,29 @@ def ports(container):
     ]).decode("utf-8").rstrip()
 
 
+def pull(image):
+    """Pull image as needed."""
+    try:
+
+        # Get digest of local image, if any
+        digest = subprocess.check_output(["docker", "inspect", "--format", "{{index .RepoDigests 0}}", image],
+                                         stderr=subprocess.DEVNULL).decode("utf-8").rstrip()
+
+        # Get digest of latest image
+        # https://stackoverflow.com/a/50945459/5156190
+        repository, tag = image.split(":") if ":" in image else image, "latest"
+        if "/" not in repository:
+            repository = "library/" + repository
+        response = requests.get(f"https://hub.docker.com/v2/repositories/{repository}/tags/{tag}").json()["images"][0]
+
+        # Pull latest if digests don't match
+        assert digest == f"{repository}@{response['digest']}"
+
+    except (AssertionError, requests.exceptions.ConnectionError, subprocess.CalledProcessError):
+
+        # Pull image
+        subprocess.call(["docker", "pull", image], stderr=subprocess.DEVNULL)
+
+
 if __name__ == "__main__":
     main()
